@@ -115,48 +115,55 @@ typedef struct _SpecialOption {
 } SpecialOption;
 
 const SpecialOption specialOptions[] = {
-    {"run", "core", FALSE,
+    {"run_key", "core", FALSE,
      METACITY "/global_keybindings/panel_run_dialog", OptionKey},
-    {"main_menu", "core", FALSE,
+    {"main_menu_key", "core", FALSE,
      METACITY "/global_keybindings/panel_main_menu", OptionKey},
-    {"run_command_screenshot", "core", FALSE,
+    {"run_command_screenshot_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_screenshot", OptionKey},
-    {"run_command_window_screenshot", "core", FALSE,
+    {"run_command_window_screenshot_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_window_screenshot", OptionKey},
-    {"run_command_terminal", "core", FALSE,
+    {"run_command_terminal_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_terminal", OptionKey},
 
-    {"toggle_window_maximized", "core", FALSE,
+    {"toggle_window_maximized_key", "core", FALSE,
      METACITY "/window_keybindings/toggle_maximized", OptionKey},
-    {"minimize_window", "core", FALSE,
+    {"minimize_window_key", "core", FALSE,
      METACITY "/window_keybindings/minimize", OptionKey},
-    {"maximize_window", "core", FALSE,
+    {"maximize_window_key", "core", FALSE,
      METACITY "/window_keybindings/maximize", OptionKey},
-    {"unmaximize_window", "core", FALSE,
+    {"unmaximize_window_key", "core", FALSE,
      METACITY "/window_keybindings/unmaximize", OptionKey},
-    {"maximize_window_horizontally", "core", FALSE,
+    {"maximize_window_horizontally_key", "core", FALSE,
      METACITY "/window_keybindings/maximize_horizontally", OptionKey},
-    {"maximize_window_vertically", "core", FALSE,
+    {"maximize_window_vertically_key", "core", FALSE,
      METACITY "/window_keybindings/maximize_vertically", OptionKey},
-    {"raise_window", "core", FALSE,
+    {"raise_window_key", "core", FALSE,
      METACITY "/window_keybindings/raise", OptionKey},
-    {"lower_window", "core", FALSE,
+    {"lower_window_key", "core", FALSE,
      METACITY "/window_keybindings/lower", OptionKey},
-    {"close_window", "core", FALSE,
+    {"close_window_key", "core", FALSE,
      METACITY "/window_keybindings/close", OptionKey},
-    {"toggle_window_shaded", "core", FALSE,
+    {"toggle_window_shaded_key", "core", FALSE,
      METACITY "/window_keybindings/toggle_shaded", OptionKey},
 
-    {"show_desktop", "core", FALSE,
+    {"show_desktop_key", "core", FALSE,
      METACITY "/global_keybindings/show_desktop", OptionKey},
 
-    {"initiate", "move", FALSE,
+    {"initiate_key", "move", FALSE,
+     METACITY "/window_keybindings/begin_move", OptionKey},
+    {"initiate_key", "resize", FALSE,
+     METACITY "/window_keybindings/begin_resize", OptionKey},
+    {"window_menu_key", "core", FALSE,
+     METACITY "/window_keybindings/activate_window_menu", OptionKey},
+
+    /* integration of Metacity's mouse_button_modifier option */
+    {"initiate_button", "move", FALSE,
      METACITY "/window_keybindings/begin_move", OptionSpecial},
-    {"initiate", "resize", FALSE,
+    {"initiate_button", "resize", FALSE,
      METACITY "/window_keybindings/begin_resize", OptionSpecial},
-    {"window_menu", "core", FALSE,
+    {"window_menu_button", "core", FALSE,
      METACITY "/window_keybindings/activate_window_menu", OptionSpecial},
-    /* this option does not exist in Compiz */
     {"mouse_button_modifier", NULL, FALSE,
      METACITY "/general/mouse_button_modifier", OptionSpecial},
 
@@ -165,14 +172,14 @@ const SpecialOption specialOptions[] = {
     {"fullscreen_visual_bell", "fade", TRUE,
      METACITY "/general/visual_bell_type", OptionSpecial},
 
-    {"next", "switcher", FALSE,
+    {"next_key", "switcher", FALSE,
      METACITY "/global_keybindings/switch_windows", OptionKey},
-    {"prev", "switcher", FALSE,
+    {"prev_key", "switcher", FALSE,
      METACITY "/global_keybindings/switch_windows_backward", OptionKey},
 
-    {"toggle_sticky", "extrawm", FALSE,
+    {"toggle_sticky_key", "extrawm", FALSE,
      METACITY "/window_keybindings/toggle_on_all_workspaces", OptionKey},
-    {"toggle_fullscreen", "extrawm", FALSE,
+    {"toggle_fullscreen_key", "extrawm", FALSE,
      METACITY "/window_keybindings/toggle_fullscreen", OptionKey},
 
     {"command0", "core", FALSE,
@@ -200,173 +207,173 @@ const SpecialOption specialOptions[] = {
     {"command11", "core", FALSE,
      METACITY "/keybinding_commands/command_12", OptionString},
 
-    {"run_command0", "core", FALSE,
+    {"run_command0_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_1", OptionKey},
-    {"run_command1", "core", FALSE,
+    {"run_command1_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_2", OptionKey},
-    {"run_command2", "core", FALSE,
+    {"run_command2_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_3", OptionKey},
-    {"run_command3", "core", FALSE,
+    {"run_command3_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_4", OptionKey},
-    {"run_command4", "core", FALSE,
+    {"run_command4_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_5", OptionKey},
-    {"run_command5", "core", FALSE,
+    {"run_command5_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_6", OptionKey},
-    {"run_command6", "core", FALSE,
+    {"run_command6_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_7", OptionKey},
-    {"run_command7", "core", FALSE,
+    {"run_command7_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_8", OptionKey},
-    {"run_command8", "core", FALSE,
+    {"run_command8_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_9", OptionKey},
-    {"run_command9", "core", FALSE,
+    {"run_command9_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_10", OptionKey},
-    {"run_command10", "core", FALSE,
+    {"run_command10_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_11", OptionKey},
-    {"run_command11", "core", FALSE,
+    {"run_command11_key", "core", FALSE,
      METACITY "/global_keybindings/run_command_12", OptionKey},
 
-    {"rotate_to_1", "rotate", FALSE,
+    {"rotate_to_1_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_1", OptionKey},
-    {"rotate_to_2", "rotate", FALSE,
+    {"rotate_to_2_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_2", OptionKey},
-    {"rotate_to_3", "rotate", FALSE,
+    {"rotate_to_3_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_3", OptionKey},
-    {"rotate_to_4", "rotate", FALSE,
+    {"rotate_to_4_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_4", OptionKey},
-    {"rotate_to_5", "rotate", FALSE,
+    {"rotate_to_5_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_5", OptionKey},
-    {"rotate_to_6", "rotate", FALSE,
+    {"rotate_to_6_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_6", OptionKey},
-    {"rotate_to_7", "rotate", FALSE,
+    {"rotate_to_7_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_7", OptionKey},
-    {"rotate_to_8", "rotate", FALSE,
+    {"rotate_to_8_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_8", OptionKey},
-    {"rotate_to_9", "rotate", FALSE,
+    {"rotate_to_9_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_9", OptionKey},
-    {"rotate_to_10", "rotate", FALSE,
+    {"rotate_to_10_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_10", OptionKey},
-    {"rotate_to_11", "rotate", FALSE,
+    {"rotate_to_11_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_11", OptionKey},
-    {"rotate_to_12", "rotate", FALSE,
+    {"rotate_to_12_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_12", OptionKey},
 
-    {"rotate_left", "rotate", FALSE,
+    {"rotate_left_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_left", OptionKey},
-    {"rotate_right", "rotate", FALSE,
+    {"rotate_right_key", "rotate", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_right", OptionKey},
 
-    {"plane_to_1", "plane", FALSE,
+    {"plane_to_1_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_1", OptionKey},
-    {"plane_to_2", "plane", FALSE,
+    {"plane_to_2_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_2", OptionKey},
-    {"plane_to_3", "plane", FALSE,
+    {"plane_to_3_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_3", OptionKey},
-    {"plane_to_4", "plane", FALSE,
+    {"plane_to_4_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_4", OptionKey},
-    {"plane_to_5", "plane", FALSE,
+    {"plane_to_5_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_5", OptionKey},
-    {"plane_to_6", "plane", FALSE,
+    {"plane_to_6_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_6", OptionKey},
-    {"plane_to_7", "plane", FALSE,
+    {"plane_to_7_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_7", OptionKey},
-    {"plane_to_8", "plane", FALSE,
+    {"plane_to_8_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_8", OptionKey},
-    {"plane_to_9", "plane", FALSE,
+    {"plane_to_9_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_9", OptionKey},
-    {"plane_to_10", "plane", FALSE,
+    {"plane_to_10_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_10", OptionKey},
-    {"plane_to_11", "plane", FALSE,
+    {"plane_to_11_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_11", OptionKey},
-    {"plane_to_12", "plane", FALSE,
+    {"plane_to_12_key", "plane", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_12", OptionKey},
 
-    {"switch_to_1", "vpswitch", FALSE,
+    {"switch_to_1_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_1", OptionKey},
-    {"switch_to_2", "vpswitch", FALSE,
+    {"switch_to_2_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_2", OptionKey},
-    {"switch_to_3", "vpswitch", FALSE,
+    {"switch_to_3_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_3", OptionKey},
-    {"switch_to_4", "vpswitch", FALSE,
+    {"switch_to_4_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_4", OptionKey},
-    {"switch_to_5", "vpswitch", FALSE,
+    {"switch_to_5_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_5", OptionKey},
-    {"switch_to_6", "vpswitch", FALSE,
+    {"switch_to_6_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_6", OptionKey},
-    {"switch_to_7", "vpswitch", FALSE,
+    {"switch_to_7_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_7", OptionKey},
-    {"switch_to_8", "vpswitch", FALSE,
+    {"switch_to_8_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_8", OptionKey},
-    {"switch_to_9", "vpswitch", FALSE,
+    {"switch_to_9_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_9", OptionKey},
-    {"switch_to_10", "vpswitch", FALSE,
+    {"switch_to_10_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_10", OptionKey},
-    {"switch_to_11", "vpswitch", FALSE,
+    {"switch_to_11_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_11", OptionKey},
-    {"switch_to_12", "vpswitch", FALSE,
+    {"switch_to_12_key", "vpswitch", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_12", OptionKey},
 
-    {"up", "wall", FALSE,
+    {"up_key", "wall", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_up", OptionKey},
-    {"down", "wall", FALSE,
+    {"down_key", "wall", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_down", OptionKey},
-    {"left", "wall", FALSE,
+    {"left_key", "wall", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_left", OptionKey},
-    {"right", "wall", FALSE,
+    {"right_key", "wall", FALSE,
      METACITY "/global_keybindings/switch_to_workspace_right", OptionKey},
-    {"left_window", "wall", FALSE,
+    {"left_window_key", "wall", FALSE,
      METACITY "/window_keybindings/move_to_workspace_left", OptionKey},
-    {"right_window", "wall", FALSE,
+    {"right_window_key", "wall", FALSE,
      METACITY "/window_keybindings/move_to_workspace_right", OptionKey},
-    {"up_window", "wall", FALSE,
+    {"up_window_key", "wall", FALSE,
      METACITY "/window_keybindings/move_to_workspace_up", OptionKey},
-    {"down_window", "wall", FALSE,
+    {"down_window_key", "wall", FALSE,
      METACITY "/window_keybindings/move_to_workspace_down", OptionKey},
 
-    {"put_topleft", "put", FALSE,
+    {"put_topleft_key", "put", FALSE,
      METACITY "/window_keybindings/move_to_corner_nw", OptionKey},
-    {"put_topright", "put", FALSE,
+    {"put_topright_key", "put", FALSE,
      METACITY "/window_keybindings/move_to_corner_ne", OptionKey},
-    {"put_bottomleft", "put", FALSE,
+    {"put_bottomleft_key", "put", FALSE,
      METACITY "/window_keybindings/move_to_corner_sw", OptionKey},
-    {"put_bottomright", "put", FALSE,
+    {"put_bottomright_key", "put", FALSE,
      METACITY "/window_keybindings/move_to_corner_se", OptionKey},
-    {"put_left", "put", FALSE,
+    {"put_left_key", "put", FALSE,
      METACITY "/window_keybindings/move_to_side_w", OptionKey},
-    {"put_right", "put", FALSE,
+    {"put_right_key", "put", FALSE,
      METACITY "/window_keybindings/move_to_side_e", OptionKey},
-    {"put_top", "put", FALSE,
+    {"put_top_key", "put", FALSE,
      METACITY "/window_keybindings/move_to_side_n", OptionKey},
-    {"put_bottom", "put", FALSE,
+    {"put_bottom_key", "put", FALSE,
      METACITY "/window_keybindings/move_to_side_s", OptionKey},
 
-    {"rotate_to_1_window", "rotate", FALSE,
+    {"rotate_to_1_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_1", OptionKey},
-    {"rotate_to_2_window", "rotate", FALSE,
+    {"rotate_to_2_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_2", OptionKey},
-    {"rotate_to_3_window", "rotate", FALSE,
+    {"rotate_to_3_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_3", OptionKey},
-    {"rotate_to_4_window", "rotate", FALSE,
+    {"rotate_to_4_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_4", OptionKey},
-    {"rotate_to_5_window", "rotate", FALSE,
+    {"rotate_to_5_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_5", OptionKey},
-    {"rotate_to_6_window", "rotate", FALSE,
+    {"rotate_to_6_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_6", OptionKey},
-    {"rotate_to_7_window", "rotate", FALSE,
+    {"rotate_to_7_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_7", OptionKey},
-    {"rotate_to_8_window", "rotate", FALSE,
+    {"rotate_to_8_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_8", OptionKey},
-    {"rotate_to_9_window", "rotate", FALSE,
+    {"rotate_to_9_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_9", OptionKey},
-    {"rotate_to_10_window", "rotate", FALSE,
+    {"rotate_to_10_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_10", OptionKey},
-    {"rotate_to_11_window", "rotate", FALSE,
+    {"rotate_to_11_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_11", OptionKey},
-    {"rotate_to_12_window", "rotate", FALSE,
+    {"rotate_to_12_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_12", OptionKey},
 
-    {"rotate_left_window", "rotate", FALSE,
+    {"rotate_left_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_left", OptionKey},
-    {"rotate_right_window", "rotate", FALSE,
+    {"rotate_right_window_key", "rotate", FALSE,
      METACITY "/window_keybindings/move_to_workspace_right", OptionKey},
 
     {"command_screenshot", "core", FALSE,
@@ -499,34 +506,6 @@ valueChanged (GConfClient *client,
 	return;
 
     setting = ccsFindSetting (plugin, token, isScreen, screenNum);
-    if (!setting)
-    {
-	/* maybe it's an action which has a name_button/... naming scheme */
-	const char *prefix[] = { "_key", "_button", "_edge", "_edgebutton", "_bell" };
-	int        i;
-	int        prefixLen, len = strlen (token);
-
-	for (i = 0; i < sizeof (prefix) / sizeof (prefix[0]); i++)
-	{
-	    prefixLen = strlen (prefix[i]);
-	    if (len < prefixLen)
-		continue;
-
-	    if (strcmp (token + len - prefixLen, prefix[i]) == 0)
-	    {
-		char *buffer = malloc (sizeof (char) * (len - prefixLen + 1));
-		if (buffer)
-		{
-		    strncpy (buffer, token, len - prefixLen);
-		    buffer[len - prefixLen] = 0;
-		    setting = ccsFindSetting (plugin, buffer,
-					      isScreen, screenNum);
-		    free (buffer);
-		}
-		break;
-	    }
-	}
-    }
     if (!setting)
 	return;
 
@@ -585,15 +564,18 @@ gnomeValueChanged (GConfClient *client,
 		needInit = FALSE;
 	    }
 
-	    s = findDisplaySettingForPlugin (context, "core", "window_menu");
+	    s = findDisplaySettingForPlugin (context, "core",
+					     "window_menu_button");
 	    if (s)
 		readSetting (context, s);
 
-	    s = findDisplaySettingForPlugin (context, "move", "initiate");
+	    s = findDisplaySettingForPlugin (context, "move",
+					     "initiate_button");
 	    if (s)
 		readSetting (context, s);
 
-	    s = findDisplaySettingForPlugin (context, "resize", "initiate");
+	    s = findDisplaySettingForPlugin (context, "resize",
+					     "initiate_button");
 	    if (s)
 		readSetting (context, s);
 	}
@@ -824,128 +806,6 @@ copyGconfTree (CCSContext  *context,
     initClient (context);
 }
 
-static Bool
-readActionValue (CCSSetting *setting,
-		 char       *pathName)
-{
-    char                  itemPath[BUFSIZE];
-    GError                *err = NULL;
-    Bool                  ret = FALSE;
-    GConfValue            *gconfValue;
-    CCSSettingActionValue action;
-
-    memset (&action, 0, sizeof (CCSSettingActionValue));
-
-    snprintf (itemPath, 512, "%s_bell", pathName);
-    gconfValue = gconf_client_get (client, itemPath, &err);
-    if (!err && gconfValue)
-    {
-	action.onBell = gconf_value_get_bool (gconfValue);
-	ret = TRUE;
-    }
-    if (err)
-    {
-	g_error_free (err);
-	err = NULL;
-    }
-    if (gconfValue)
-	gconf_value_free (gconfValue);
-
-    snprintf (itemPath, 512, "%s_edge", pathName);
-    gconfValue = gconf_client_get (client, itemPath, &err);
-    if (!err && gconfValue)
-    {
-	if ((gconfValue->type == GCONF_VALUE_LIST) &&
-	    (gconf_value_get_list_type (gconfValue) == GCONF_VALUE_STRING))
-	{
-	    GSList        *list;
-	    CCSStringList edgeList = NULL;
-
-	    list = gconf_value_get_list (gconfValue);
-	    for ( ; list; list = list->next)
-	    {
-		GConfValue *value = (GConfValue *) list->data;
-		const char *edge = gconf_value_get_string (value);
-
-		if (edge)
-		    edgeList = ccsStringListAppend (edgeList, (char*) edge);
-	    }
-
-	    ccsStringListToEdges (edgeList, &action);
-
-	    if (edgeList)
-		ccsStringListFree (edgeList, FALSE);
-	}
-    }
-    if (err)
-    {
-	g_error_free (err);
-	err = NULL;
-    }
-    if (gconfValue)
-	gconf_value_free (gconfValue);
-
-    snprintf (itemPath, 512, "%s_edgebutton", pathName);
-    gconfValue = gconf_client_get (client, itemPath, &err);
-    if (!err && gconfValue)
-    {
-	action.edgeButton = gconf_value_get_int (gconfValue);
-	ret = TRUE;
-    }
-    if (err)
-    {
-	g_error_free (err);
-	err = NULL;
-    }
-    if (gconfValue)
-	gconf_value_free (gconfValue);
-
-    snprintf (itemPath, 512, "%s_key", pathName);
-    gconfValue = gconf_client_get (client, itemPath, &err);
-    if (!err && gconfValue)
-    {
-	const char* buffer;
-	buffer = gconf_value_get_string (gconfValue);
-	if (buffer)
-	{
-	    ccsStringToKeyBinding (buffer, &action);
-	    ret = TRUE;
-	}
-    }
-    if (err)
-    {
-	g_error_free (err);
-	err = NULL;
-    }
-    if (gconfValue)
-	gconf_value_free (gconfValue);
-
-    snprintf (itemPath, 512, "%s_button", pathName);
-    gconfValue = gconf_client_get (client, itemPath, &err);
-    if (!err && gconfValue)
-    {
-	const char* buffer;
-	buffer = gconf_value_get_string (gconfValue);
-	if (buffer)
-	{
-	    ccsStringToButtonBinding (buffer, &action);
-	    ret = TRUE;
-	}
-    }
-    if (err)
-    {
-	g_error_free (err);
-	err = NULL;
-    }
-    if (gconfValue)
-	gconf_value_free (gconfValue);
-
-    if (ret)
-	ccsSetAction (setting, action);
-
-    return ret;
-}
-
 static Bool
 readListValue (CCSSetting *setting,
 	       GConfValue *gconfValue)
@@ -1108,80 +968,74 @@ readIntegratedOption (CCSContext *context,
 		      CCSSetting *setting,
 		      int        index)
 {
-    GError *err = NULL;
-    Bool   ret = FALSE;
+    GConfValue *gconfValue;
+    GError     *err = NULL;
+    Bool       ret = FALSE;
 
-    switch (specialOptions[index].type)
+    gconfValue = gconf_client_get (client,
+				   specialOptions[index].gnomeName,
+				   &err);
+
+    if (err)
     {
+	g_error_free (err);
+	return FALSE;
+    }
+
+    if (!gconfValue)
+	return FALSE;
+
+    switch (specialOptions[index].type) {
     case OptionInt:
+	if (gconfValue->type == GCONF_VALUE_INT)
 	{
 	    guint value;
-	    value = gconf_client_get_int (client,
-					  specialOptions[index].gnomeName,
-					  &err);
 
-	    if (!err)
-	    {
-		ccsSetInt (setting, value);
-		ret = TRUE;
-	    }
+	    value = gconf_value_get_int (gconfValue);
+	    ccsSetInt (setting, value);
+	    ret = TRUE;
 	}
 	break;
     case OptionBool:
+	if (gconfValue->type == GCONF_VALUE_BOOL)
 	{
 	    gboolean value;
-	    value = gconf_client_get_bool (client,
-					   specialOptions[index].gnomeName,
-					   &err);
 
-	    if (!err)
-	    {
-		ccsSetBool (setting, value ? TRUE : FALSE);
-		ret = TRUE;
-	    }
+	    value = gconf_value_get_bool (gconfValue);
+	    ccsSetBool (setting, value ? TRUE : FALSE);
+	    ret = TRUE;
 	}
 	break;
     case OptionString:
+	if (gconfValue->type == GCONF_VALUE_STRING)
 	{
-	    char *value;
-	    value = gconf_client_get_string (client,
-					     specialOptions[index].gnomeName,
-					     &err);
+	    const char *value;
 
-	    if (!err && value)
-    	    {
+	    value = gconf_value_get_string (gconfValue);
+	    if (value)
+	    {
 		ccsSetString (setting, value);
 		ret = TRUE;
-		g_free (value);
 	    }
 	}
 	break;
     case OptionKey:
+	if (gconfValue->type == GCONF_VALUE_STRING)
 	{
-	    char *value;
-	    value = gconf_client_get_string (client,
-					     specialOptions[index].gnomeName,
-					     &err);
+	    const char *value;
 
-	    if (!err && value)
-    	    {
-		CCSSettingActionValue action;
-		memset (&action, 0, sizeof(CCSSettingActionValue));
-		ccsGetAction (setting, &action);
-		if (strcasecmp (value, "disabled") == 0)
+	    value = gconf_value_get_string (gconfValue);
+	    if (value)
+	    {
+		CCSSettingKeyValue key;
+
+		memset (&key, 0, sizeof (CCSSettingKeyValue));
+		ccsGetKey (setting, &key);
+		if (ccsStringToKeyBinding (value, &key))
 		{
-		    /* clear out keybinding */
-		    action.keysym     = NoSymbol;
-		    action.keyModMask = 0;
-		    ccsSetAction (setting, action);
+		    ccsSetKey (setting, key);
 		    ret = TRUE;
 		}
-		else if (ccsStringToKeyBinding (value, &action))
-		{
-		    ccsSetAction (setting, action);
-		    ret = TRUE;
-		}
-		g_free (value);
 	    }
 	}
 	break;
@@ -1192,98 +1046,76 @@ readIntegratedOption (CCSContext *context,
 
 	    if (strcmp (settingName, "current_viewport") == 0)
 	    {
-		const char *name;
-		gboolean   showAll;
-
-		name = specialOptions[index].gnomeName;
-		showAll = gconf_client_get_bool (client, name, &err);
-		if (!err)
+		if (gconfValue->type == GCONF_VALUE_BOOL)
 		{
+		    gboolean showAll;
+
+		    showAll = gconf_value_get_bool (gconfValue);
 		    ccsSetBool (setting, !showAll);
 		    ret = TRUE;
 		}
 	    }
 	    else if (strcmp (settingName, "fullscreen_visual_bell") == 0)
 	    {
-		const char *name;
-		char       *value;
-
-		name = specialOptions[index].gnomeName;
-		value = gconf_client_get_string (client, name, &err);
-		if (!err && value)
+		if (gconfValue->type == GCONF_VALUE_STRING)
 		{
-		    Bool fullscreen;
+		    const char *value;
 
-		    fullscreen = strcmp (value, "fullscreen") == 0;
-		    ccsSetBool (setting, fullscreen);
-		    ret = TRUE;
-		    g_free (value);
+		    value = gconf_value_get_string (gconfValue);
+		    if (value)
+		    {
+			Bool fullscreen;
+
+			fullscreen = strcmp (value, "fullscreen") == 0;
+			ccsSetBool (setting, fullscreen);
+			ret = TRUE;
+		    }
 		}
 	    }
 	    else if (strcmp (settingName, "click_to_focus") == 0)
 	    {
-		char       *focusMode;
-		const char *name;
-
-		name = specialOptions[index].gnomeName;
-		focusMode = gconf_client_get_string (client, name, &err);
-
-		if (!err && focusMode)
+		if (gconfValue->type == GCONF_VALUE_STRING)
 		{
-		    Bool clickToFocus = (strcmp (focusMode, "click") == 0);
-		    ccsSetBool (setting, clickToFocus);
-		    ret = TRUE;
-		    g_free (focusMode);
-		}
-	    }
-	    else if (((strcmp (settingName, "initiate") == 0) &&
-		      ((strcmp (pluginName, "move") == 0) ||
-		       (strcmp (pluginName, "resize") == 0))) ||
-		     (strcmp (settingName, "window_menu") == 0))
-	    {
-		char       *value;
-		const char *name;
+		    const char *focusMode;
 
-		name = specialOptions[index].gnomeName;
-		value = gconf_client_get_string (client, name, &err);
+		    focusMode = gconf_value_get_string (gconfValue);
 
-		if (!err && value)
-		{
-		    CCSSettingActionValue action;
-		    memset (&action, 0, sizeof(CCSSettingActionValue));
-		    ccsGetAction (setting, &action);
-
-		    if (strcasecmp (value, "disabled") == 0)
+		    if (focusMode)
 		    {
-			/* clear out keybinding */
-			action.keysym     = NoSymbol;
-			action.keyModMask = 0;
-		    }
-		    if ((strcasecmp (value, "disabled") == 0) ||
-			ccsStringToKeyBinding (value, &action))
-		    {
-			action.buttonModMask = getGnomeMouseButtonModifier ();
-			if (strcmp (settingName, "window_menu") == 0)
-			    action.button = 3;
-			else if (strcmp (pluginName, "resize") == 0)
-			    action.button = 2;
-			else
-			    action.button = 1;
-
-			ccsSetAction (setting, action);
+			Bool clickToFocus = (strcmp (focusMode, "click") == 0);
+			ccsSetBool (setting, clickToFocus);
 			ret = TRUE;
 		    }
-		    g_free (value);
 		}
 	    }
+	    else if (((strcmp (settingName, "initiate_button") == 0) &&
+		      ((strcmp (pluginName, "move") == 0) ||
+		       (strcmp (pluginName, "resize") == 0))) ||
+		      ((strcmp (settingName, "window_menu_button") == 0) &&
+		       (strcmp (pluginName, "core") == 0)))
+	    {
+		CCSSettingButtonValue button;
+		memset (&button, 0, sizeof (CCSSettingButtonValue));
+		ccsGetButton (setting, &button);
+
+		button.buttonModMask = getGnomeMouseButtonModifier ();
+		if (strcmp (settingName, "window_menu_button") == 0)
+		    button.button = 3;
+		else if (strcmp (pluginName, "resize") == 0)
+		    button.button = 2;
+		else
+		    button.button = 1;
+
+		ccsSetButton (setting, button);
+		ret = TRUE;
+	    }
 	}
      	break;
     default:
 	break;
     }
 
-    if (err)
-	g_error_free (err);
+    gconf_value_free (gconfValue);
 
     return ret;
 }
@@ -1294,57 +1126,56 @@ readOption (CCSSetting * setting)
     GConfValue *gconfValue = NULL;
     GError     *err = NULL;
     Bool       ret = FALSE;
+    Bool       valid = TRUE;
 
     KEYNAME;
     PATHNAME;
 
-    /* first check if the key is set, but only if the setting
-       type is not action - actions are in a subtree and handled
-       separately */
-    if (setting->type != TypeAction)
+    /* first check if the key is set */
+    gconfValue = gconf_client_get_without_default (client, pathName, &err);
+    if (err)
     {
-	Bool valid = TRUE;
-	gconfValue = gconf_client_get_without_default (client, pathName, &err);
-	if (err)
-	{
-	    g_error_free (err);
-	    return FALSE;
-	}
-	if (!gconfValue)
-	    /* value is not set */
-	    return FALSE;
+	g_error_free (err);
+	return FALSE;
+    }
+    if (!gconfValue)
+	/* value is not set */
+	return FALSE;
 
-	/* setting type sanity check */
-	switch (setting->type)
-	{
-	case TypeString:
-    	case TypeMatch:
-	case TypeColor:
-	    valid = (gconfValue->type == GCONF_VALUE_STRING);
-	    break;
-	case TypeInt:
-	    valid = (gconfValue->type == GCONF_VALUE_INT);
-	    break;
-	case TypeBool:
-	    valid = (gconfValue->type == GCONF_VALUE_BOOL);
-	    break;
-	case TypeFloat:
-	    valid = (gconfValue->type == GCONF_VALUE_FLOAT);
-	    break;
-	case TypeList:
-	    valid = (gconfValue->type == GCONF_VALUE_LIST);
-	    break;
-	default:
-	    break;
-	}
-	if (!valid)
-	{
-	    printf ("GConf backend: There is an unsupported value at path %s. "
-		    "Settings from this path won't be read. Try to remove "
-		    "that value so that operation can continue properly.\n",
-		    pathName);
-	    return FALSE;
-	}
+    /* setting type sanity check */
+    switch (setting->type)
+    {
+    case TypeString:
+    case TypeMatch:
+    case TypeColor:
+    case TypeKey:
+    case TypeButton:
+    case TypeEdge:
+	valid = (gconfValue->type == GCONF_VALUE_STRING);
+	break;
+    case TypeInt:
+	valid = (gconfValue->type == GCONF_VALUE_INT);
+	break;
+    case TypeBool:
+    case TypeBell:
+	valid = (gconfValue->type == GCONF_VALUE_BOOL);
+	break;
+    case TypeFloat:
+	valid = (gconfValue->type == GCONF_VALUE_FLOAT);
+	break;
+    case TypeList:
+	valid = (gconfValue->type == GCONF_VALUE_LIST);
+	break;
+    default:
+	break;
+    }
+    if (!valid)
+    {
+	printf ("GConf backend: There is an unsupported value at path %s. "
+		"Settings from this path won't be read. Try to remove "
+		"that value so that operation can continue properly.\n",
+		pathName);
+	return FALSE;
     }
 
     switch (setting->type)
@@ -1411,12 +1242,58 @@ readOption (CCSSetting * setting)
 	    }
 	}
 	break;
+    case TypeKey:
+	{
+	    const char         *value;
+	    CCSSettingKeyValue key;
+	    value = gconf_value_get_string (gconfValue);
+
+	    if (value && ccsStringToKeyBinding (value, &key))
+	    {
+		ccsSetKey (setting, key);
+		ret = TRUE;
+	    }
+	}
+	break;
+    case TypeButton:
+	{
+	    const char            *value;
+	    CCSSettingButtonValue button;
+	    value = gconf_value_get_string (gconfValue);
+
+	    if (value && ccsStringToButtonBinding (value, &button))
+	    {
+		ccsSetButton (setting, button);
+		ret = TRUE;
+	    }
+	}
+	break;
+    case TypeEdge:
+	{
+	    const char   *value;
+	    value = gconf_value_get_string (gconfValue);
+
+	    if (value)
+	    {
+		unsigned int edges;
+		edges = ccsStringToEdges (value);
+		ccsSetEdge (setting, edges);
+		ret = TRUE;
+	    }
+	}
+	break;
+    case TypeBell:
+	{
+	    gboolean value;
+	    value = gconf_value_get_bool (gconfValue);
+
+	    ccsSetBell (setting, value ? TRUE : FALSE);
+	    ret = TRUE;
+	}
+	break;
     case TypeList:
 	ret = readListValue (setting, gconfValue);
 	break;
-    case TypeAction:
-	ret = readActionValue (setting, pathName);
-	break;
     default:
 	printf("GConf backend: attempt to read unsupported setting type %d!\n",
 	       setting->type);
@@ -1429,49 +1306,6 @@ readOption (CCSSetting * setting)
     return ret;
 }
 
-static void
-writeActionValue(CCSSettingActionValue *action,
-		 char                  *pathName)
-{
-    char          *buffer;
-    char          itemPath[BUFSIZE];
-    CCSStringList edgeList, l;
-    GSList        *list = NULL;
-
-    snprintf (itemPath, BUFSIZE, "%s_edge", pathName);
-    edgeList = ccsEdgesToStringList (action);
-    for (l = edgeList; l; l = l->next)
-	list = g_slist_append (list, l->data);
-
-    gconf_client_set_list (client, itemPath, GCONF_VALUE_STRING, list, NULL);
-    if (edgeList)
-	ccsStringListFree (edgeList, TRUE);
-    if (list)
-	g_slist_free (list);
-
-    snprintf (itemPath, BUFSIZE, "%s_bell", pathName);
-    gconf_client_set_bool (client, itemPath, action->onBell, NULL);
-
-    snprintf (itemPath, BUFSIZE, "%s_edgebutton", pathName);
-    gconf_client_set_int (client, itemPath, action->edgeButton, NULL);
-
-    snprintf (itemPath, BUFSIZE, "%s_button", pathName);
-    buffer = ccsButtonBindingToString (action);
-    if (buffer)
-    {
-	gconf_client_set_string (client, itemPath, buffer, NULL);
-	free (buffer);
-    }
-
-    snprintf (itemPath, BUFSIZE, "%s_key", pathName);
-    buffer = ccsKeyBindingToString (action);
-    if (buffer)
-    {
-	gconf_client_set_string (client, itemPath, buffer, NULL);
-	free (buffer);
-    }
-}
-
 static void
 writeListValue (CCSSetting *setting,
 		char       *pathName)
@@ -1626,23 +1460,23 @@ setButtonBindingForSetting (CCSContext   *context,
 			    unsigned int buttonModMask)
 {
     CCSSetting            *s;
-    CCSSettingActionValue action;
+    CCSSettingButtonValue value;
 
     s = findDisplaySettingForPlugin (context, (char*) plugin, (char*) setting);
     if (!s)
 	return;
 
-    if (s->type != TypeAction)
+    if (s->type != TypeButton)
 	return;
 
-    action = s->value->value.asAction;
+    value = s->value->value.asButton;
 
-    if ((action.button != button) || (action.buttonModMask != buttonModMask))
+    if ((value.button != button) || (value.buttonModMask != buttonModMask))
     {
-	action.button = button;
-	action.buttonModMask = buttonModMask;
+	value.button = button;
+	value.buttonModMask = buttonModMask;
 
-	ccsSetAction (s, action);
+	ccsSetButton (s, value);
     }
 }
 
@@ -1704,18 +1538,17 @@ writeIntegratedOption (CCSContext *context,
 	    char  *newValue;
 	    gchar *currentValue;
 
-	    newValue = ccsKeyBindingToString (&setting->value->value.asAction);
-	    if (newValue && !strlen (newValue))
-	    {
-		free (newValue);
-		newValue = NULL;
-	    }
-	    if (!newValue)
-		newValue = strdup ("disabled");
+	    newValue = ccsKeyBindingToString (&setting->value->value.asKey);
 	    if (newValue)
     	    {
-		currentValue = gconf_client_get_string(client,
-						       optionName, &err);
+		if (strcmp (newValue, "Disabled") == 0)
+		{
+		    /* Metacity doesn't like "Disabled", it wants "disabled" */
+		    newValue[0] = 'd';
+		}
+
+		currentValue = gconf_client_get_string (client,
+							optionName, &err);
 
 		if (!err && currentValue)
 		{
@@ -1780,47 +1613,22 @@ writeIntegratedOption (CCSContext *context,
 		    g_free (currentValue);
 		}
 	    }
-	    else if (((strcmp (settingName, "initiate") == 0) &&
-	    	      ((strcmp (pluginName, "move") == 0) ||
+	    else if (((strcmp (settingName, "initiate_button") == 0) &&
+		      ((strcmp (pluginName, "move") == 0) ||
 		       (strcmp (pluginName, "resize") == 0))) ||
-		     (strcmp (settingName, "window_menu") == 0))
+		      ((strcmp (settingName, "window_menu_button") == 0) &&
+		       (strcmp (pluginName, "core") == 0)))
 	    {
-		char         *newValue;
-		gchar        *currentValue;
 		unsigned int modMask;
 
-		modMask = setting->value->value.asAction.buttonModMask;
-		newValue =
-		    ccsKeyBindingToString (&setting->value->value.asAction);
-		if (newValue && !strlen (newValue))
-		{
-		    free (newValue);
-		    newValue = NULL;
-		}
-		if (!newValue)
-		    newValue = strdup ("disabled");
-		if (newValue)
-		{
-		    currentValue = gconf_client_get_string(client,
-							   optionName, &err);
-
-		    if (!err && currentValue)
-		    {
-			if (strcmp (currentValue, newValue) != 0)
-    			    gconf_client_set_string(client, optionName,
-						    newValue, NULL);
-			g_free (currentValue);
-    		    }
-		    free (newValue);
-		}
-
+		modMask = setting->value->value.asButton.buttonModMask;
 		setGnomeMouseButtonModifier (modMask);
 		setButtonBindingForSetting (context, "move",
-					    "initiate", 1, modMask);
+					    "initiate_button", 1, modMask);
 		setButtonBindingForSetting (context, "resize",
-					    "initiate", 2, modMask);
+					    "initiate_button", 2, modMask);
 		setButtonBindingForSetting (context, "core",
-					    "window_menu", 3, modMask);
+					    "window_menu_button", 3, modMask);
 	    }
 	}
      	break;
@@ -1836,28 +1644,7 @@ resetOptionToDefault (CCSSetting * setting)
     KEYNAME;
     PATHNAME;
 
-    if (setting->type != TypeAction)
-	gconf_client_recursive_unset (client, pathName, 0, NULL);
-    else
-    {
-	char itemPath[BUFSIZE];
-
-	snprintf (itemPath, BUFSIZE, "%s_edge", pathName);
-	gconf_client_recursive_unset (client, itemPath, 0, NULL);
-
-	snprintf (itemPath, BUFSIZE, "%s_edgebutton", pathName);
-	gconf_client_recursive_unset (client, itemPath, 0, NULL);
-
-	snprintf (itemPath, BUFSIZE, "%s_button", pathName);
-	gconf_client_recursive_unset (client, itemPath, 0, NULL);
-
-	snprintf (itemPath, BUFSIZE, "%s_key", pathName);
-	gconf_client_recursive_unset (client, itemPath, 0, NULL);
-
-	snprintf (itemPath, BUFSIZE, "%s_bell", pathName);
-	gconf_client_recursive_unset (client, itemPath, 0, NULL);
-    }
-
+    gconf_client_recursive_unset (client, pathName, 0, NULL);
     gconf_client_suggest_sync (client, NULL);
 }
 
@@ -1919,13 +1706,59 @@ writeOption (CCSSetting * setting)
 	    free (colString);
 	}
 	break;
-    case TypeAction:
+    case TypeKey:
 	{
-	    CCSSettingActionValue value;
-	    if (!ccsGetAction (setting, &value))
+	    CCSSettingKeyValue key;
+	    char               *keyString;
+
+	    if (!ccsGetKey (setting, &key))
 		break;
 
-	    writeActionValue (&value, pathName);
+	    keyString = ccsKeyBindingToString (&key);
+	    if (!keyString)
+		break;
+
+	    gconf_client_set_string (client, pathName, keyString, NULL);
+	    free (keyString);
+	}
+	break;
+    case TypeButton:
+	{
+	    CCSSettingButtonValue button;
+	    char                  *buttonString;
+
+	    if (!ccsGetButton (setting, &button))
+		break;
+
+	    buttonString = ccsButtonBindingToString (&button);
+	    if (!buttonString)
+		break;
+
+	    gconf_client_set_string (client, pathName, buttonString, NULL);
+	    free (buttonString);
+	}
+	break;
+    case TypeEdge:
+	{
+	    unsigned int edges;
+	    char         *edgeString;
+
+	    if (!ccsGetEdge (setting, &edges))
+		break;
+
+	    edgeString = ccsEdgesToString (edges);
+	    if (!edgeString)
+		break;
+
+	    gconf_client_set_string (client, pathName, edgeString, NULL);
+	    free (edgeString);
+	}
+	break;
+    case TypeBell:
+	{
+	    Bool value;
+	    if (ccsGetBell (setting, &value))
+		gconf_client_set_bool (client, pathName, value, NULL);
 	}
 	break;
     case TypeList:
